@@ -1,69 +1,29 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qv4globalobject_p.h"
+
+#include <private/qv4alloca_p.h>
+#include <private/qv4codegen_p.h>
+#include <private/qv4context_p.h>
+#include <private/qv4function_p.h>
 #include <private/qv4mm_p.h>
-#include "qv4value_p.h"
-#include "qv4context_p.h"
-#include "qv4function_p.h"
-#include "qv4debugging_p.h"
-#include "qv4profiling_p.h"
-#include "qv4script_p.h"
-#include "qv4scopedvalue_p.h"
-#include "qv4string_p.h"
-#include "qv4jscall_p.h"
-
-#include <private/qqmljsengine_p.h>
-#include <private/qqmljslexer_p.h>
-#include <private/qqmljsparser_p.h>
-#include <private/qqmljsast_p.h>
-#include <qv4codegen_p.h>
-#include "private/qlocale_tools_p.h"
-#include "private/qtools_p.h"
-
-#include <QtCore/QDebug>
-#include <QtCore/QString>
-#include <iostream>
-#include "qv4alloca_p.h"
+#include <private/qv4scopedvalue_p.h>
+#include <private/qv4script_p.h>
+#include <private/qv4stackframe_p.h>
+#include <private/qv4string_p.h>
+#include <private/qv4value_p.h>
 
 #include <wtf/MathExtras.h>
 
+#include <QtCore/private/qlocale_tools_p.h>
+#include <QtCore/private/qtools_p.h>
+
+#include <QtCore/qdebug.h>
+#include <QtCore/qstring.h>
+
+#include <iostream>
+
 using namespace QV4;
 using QtMiscUtils::toHexUpper;
 using QtMiscUtils::fromHex;
@@ -72,7 +32,7 @@ static QString escape(const QString &input)
 {
     QString output;
     output.reserve(input.size() * 3);
-    const int length = input.length();
+    const int length = input.size();
     for (int i = 0; i < length; ++i) {
         ushort uc = input.at(i).unicode();
         if (uc < 0x100) {
@@ -84,13 +44,13 @@ static QString escape(const QString &input)
                 || (uc == 0x5F)) {
                 output.append(QChar(uc));
             } else {
-                output.append('%');
+                output.append(u'%');
                 output.append(QLatin1Char(toHexUpper(uc >> 4)));
                 output.append(QLatin1Char(toHexUpper(uc)));
             }
         } else {
-            output.append('%');
-            output.append('u');
+            output.append(u'%');
+            output.append(u'u');
             output.append(QLatin1Char(toHexUpper(uc >> 12)));
             output.append(QLatin1Char(toHexUpper(uc >> 8)));
             output.append(QLatin1Char(toHexUpper(uc >> 4)));
@@ -103,14 +63,14 @@ static QString escape(const QString &input)
 static QString unescape(const QString &input)
 {
     QString result;
-    result.reserve(input.length());
+    result.reserve(input.size());
     int i = 0;
-    const int length = input.length();
+    const int length = input.size();
     while (i < length) {
         QChar c = input.at(i++);
-        if ((c == '%') && (i + 1 < length)) {
+        if ((c == u'%') && (i + 1 < length)) {
             QChar a = input.at(i);
-            if ((a == 'u') && (i + 4 < length)) {
+            if ((a == u'u') && (i + 4 < length)) {
                 int d3 = fromHex(input.at(i+1).unicode());
                 int d2 = fromHex(input.at(i+2).unicode());
                 int d1 = fromHex(input.at(i+3).unicode());
@@ -126,7 +86,7 @@ static QString unescape(const QString &input)
                 int d1 = fromHex(a.unicode());
                 int d0 = fromHex(input.at(i+1).unicode());
                 if ((d1 != -1) && (d0 != -1)) {
-                    c = (d1 << 4) | d0;
+                    c = QChar((d1 << 4) | d0);
                     i += 2;
                 }
                 result.append(c);
@@ -153,7 +113,7 @@ static QString encode(const QString &input, const char *unescapedSet, bool *ok)
 {
     *ok = true;
     QString output;
-    const int length = input.length();
+    const int length = input.size();
     int i = 0;
     while (i < length) {
         const QChar c = input.at(i);
@@ -227,8 +187,8 @@ static QString decode(const QString &input, DecodeMode decodeMode, bool *ok)
 {
     *ok = true;
     QString output;
-    output.reserve(input.length());
-    const int length = input.length();
+    output.reserve(input.size());
+    const int length = input.size();
     int i = 0;
     const QChar percent = QLatin1Char('%');
     while (i < length) {
@@ -308,7 +268,7 @@ static QString decode(const QString &input, DecodeMode decodeMode, bool *ok)
                         ++r;
                     }
                     if (*r)
-                        output.append(input.midRef(start, i - start + 1));
+                        output.append(QStringView{input}.mid(start, i - start + 1));
                     else
                         output.append(QChar(b));
                 } else {
@@ -335,7 +295,7 @@ void Heap::EvalFunction::init(QV4::ExecutionContext *scope)
     Scope s(scope);
     Heap::FunctionObject::init(scope, s.engine->id_eval());
     ScopedFunctionObject f(s, this);
-    f->defineReadonlyProperty(s.engine->id_length(), Primitive::fromInt32(1));
+    f->defineReadonlyConfigurableProperty(s.engine->id_length(), Value::fromInt32(1));
 }
 
 ReturnedValue EvalFunction::evalCall(const Value *, const Value *argv, int argc, bool directCall) const
@@ -351,7 +311,7 @@ ReturnedValue EvalFunction::evalCall(const Value *, const Value *argv, int argc,
 
     if (!directCall) {
         // the context for eval should be the global scope
-        ctx = v4->rootContext();
+        ctx = v4->scriptContext();
     }
 
     String *scode = argv[0].stringValue();
@@ -361,7 +321,7 @@ ReturnedValue EvalFunction::evalCall(const Value *, const Value *argv, int argc,
     const QString code = scode->toQString();
     bool inheritContext = !isStrict;
 
-    Script script(ctx, QV4::Compiler::EvalCode, code, QStringLiteral("eval code"));
+    Script script(ctx, QV4::Compiler::ContextType::Eval, code, QStringLiteral("eval code"));
     script.strictMode = (directCall && isStrict);
     script.inheritContext = inheritContext;
     script.parse();
@@ -371,20 +331,21 @@ ReturnedValue EvalFunction::evalCall(const Value *, const Value *argv, int argc,
     Function *function = script.function();
     if (!function)
         return Encode::undefined();
+    function->kind = Function::Eval;
 
     if (function->isStrict() || isStrict) {
         ScopedFunctionObject e(scope, FunctionObject::createScriptFunction(ctx, function));
         ScopedValue thisObject(scope, directCall ? scope.engine->currentStackFrame->thisObject() : scope.engine->globalObject->asReturnedValue());
-        return e->call(thisObject, nullptr, 0);
+        return checkedResult(v4, e->call(thisObject, nullptr, 0));
     }
 
     ScopedValue thisObject(scope, scope.engine->currentStackFrame->thisObject());
 
-    return function->call(thisObject, nullptr, 0, ctx);
+    return checkedResult(v4, function->call(thisObject, nullptr, 0, ctx));
 }
 
 
-ReturnedValue EvalFunction::call(const FunctionObject *f, const Value *thisObject, const Value *argv, int argc)
+ReturnedValue EvalFunction::virtualCall(const FunctionObject *f, const Value *thisObject, const Value *argv, int argc)
 {
     // indirect call
     return static_cast<const EvalFunction *>(f)->evalCall(thisObject, argv, argc, false);
@@ -411,8 +372,8 @@ static inline int toInt(const QChar &qc, int R)
 ReturnedValue GlobalFunctions::method_parseInt(const FunctionObject *b, const Value *, const Value *argv, int argc)
 {
     Scope scope(b);
-    ScopedValue inputString(scope, argc ? argv[0] : Primitive::undefinedValue());
-    ScopedValue radix(scope, argc > 1 ? argv[1] : Primitive::undefinedValue());
+    ScopedValue inputString(scope, argc ? argv[0] : Value::undefinedValue());
+    ScopedValue radix(scope, argc > 1 ? argv[1] : Value::undefinedValue());
     int R = radix->isUndefined() ? 0 : radix->toInt32();
 
     // [15.1.2.2] step by step:
@@ -420,7 +381,7 @@ ReturnedValue GlobalFunctions::method_parseInt(const FunctionObject *b, const Va
     CHECK_EXCEPTION();
 
     const QChar *pos = trimmed.constData();
-    const QChar *end = pos + trimmed.length();
+    const QChar *end = pos + trimmed.size();
 
     int sign = 1; // 3
     if (pos != end) {
@@ -493,7 +454,7 @@ ReturnedValue GlobalFunctions::method_parseFloat(const FunctionObject *b, const
 {
     Scope scope(b);
     // [15.1.2.3] step by step:
-    ScopedString inputString(scope, argc ? argv[0] : Primitive::undefinedValue(), ScopedString::Convert);
+    ScopedString inputString(scope, argc ? argv[0] : Value::undefinedValue(), ScopedString::Convert);
     CHECK_EXCEPTION();
 
     QString trimmed = inputString->toQString().trimmed(); // 2
